from autogen_agentchat.conditions import TextMentionTermination,MaxMessageTermination


#text_termination = TextMentionTermination("TERMINATE")
text_termination = TextMentionTermination("APPROVE")
#text_termination2 = TextMentionTermination("q") # not working
max_msg_termination = MaxMessageTermination(max_messages=100)
termination = max_msg_termination | text_termination



from autogen_agentchat.agents import AssistantAgent
from autogen_agentchat.conditions import TextMentionTermination
from autogen_ext.models.openai import OpenAIChatCompletionClient

from autowork.autowork_ModelClient import model_client



# assistant_my = AssistantAgent("assistant", model_client=model_client)

assistant = AssistantAgent(name="assistant_agent", 
                    model_client=model_client,
                    tools=[],
                    handoffs=[],
                    description="An agent that provides assistance with ability to use tools.",
                    system_message='''
                    You are a helpful assistant. Solve tasks carefully. 
                    If the output has code, just give one block of code. 
                    Do not give me multiple suggestions. When done, say TERMINATE.
                    ''',
                    model_client_stream=True)

# something is missing:
# 



from autogen_ext.models.openai import OpenAIChatCompletionClient

# model_client = OpenAIChatCompletionClient(model="gpt-4o", api_key="sk-xxx")

# ollama local deployment
def get_model_client_ollama() -> OpenAIChatCompletionClient:  # type: ignore
    return OpenAIChatCompletionClient(
        model="llama3.2:latest",
        api_key="ollama",
        base_url="http://localhost:11434/v1",
        model_capabilities={
            "json_output": True,
            "vision": False,
            "function_calling": True,
        },
    )


def custom_model_client() -> OpenAIChatCompletionClient:
    return OpenAIChatCompletionClient(
        model="custom-model-name",
        base_url="https://custom-model.com/reset/of/the/path",
        api_key="placeholder",
        model_info={
            "vision": True,
            "function_calling": True,
            "json_output": True,
            "family": "unknown",
        },
    )

def get_model_client_siliconflow() -> OpenAIChatCompletionClient:  # type: ignore
    return OpenAIChatCompletionClient(
        model="deepseek-ai/DeepSeek-V3",
        #model="deepseek-ai/DeepSeek-R1-Distill-Llama-70B",
        api_key="sk-uefrbugirppbfzegplcdszjgrfimuifcglivcqhuzzilokkv", #germany
        base_url="https://api.siliconflow.cn/v1",
        model_capabilities={
            "json_output": False,
            "vision": False,
            "function_calling": True,
        },
    )



# model_client = get_model_client_ollama()

model_client = get_model_client_siliconflow()



#from autogen_agentchat.agents import UserProxyAgent

""" from autogen_agentchat.agents import UserProxyAgent

class CustomUserProxyAgent(UserProxyAgent):
    def __init__(self, name="user", input_func=input):
        super().__init__(name=name)
        self.input_func = input_func

    async def on_input(self, prompt: str) -> str:
        user_input = input(prompt)
        if user_input.strip().lower() == 'q':
            print("Exiting the program.")
            raise SystemExit
        return user_input
"""
from autogen_agentchat.agents import UserProxyAgent

class CustomUserProxyAgent(UserProxyAgent):
    def __init__(self, name="user", description=None, input_func=input):
        super().__init__(name=name, description=description, input_func=input_func)
        self.input_func = input_func

    async def on_input(self, prompt: str) -> str:
        user_input = self.input_func(prompt)
        if user_input.strip().lower() == 'q':
            print("Exiting the program.")
            raise SystemExit
        return user_input


#user_proxy = UserProxyAgent(name = "user_proxy", input_func=input,
#                            description="An agent that can represent a human user through an input function.",
#                            )
        
user_proxy = CustomUserProxyAgent(name = "user_proxy", input_func=input,
                description="An agent that can represent a human user through an input function.",
)
 


from setuptools import setup, find_packages

setup(
    name="autowork",
    version="0.4.1.2",
    packages=find_packages(),
    install_requires=[
        # List dependencies here, e.g.,
        # "requests",
        # "numpy"
    ],
    entry_points={
        "console_scripts": [
            "autowork = autowork.autowork:main",
        ],
    },
    author="Yingjie Wei",
    author_email="yingjie.wei@qq.com",
    description="autowork to automatically process the task (write code and execute it) based on LLM.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yingjiewei/autowork",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



from autogen_agentchat.agents import CodeExecutorAgent
from autogen_ext.code_executors.local import LocalCommandLineCodeExecutor
from pathlib import Path

work_dir = Path("coding")
work_dir.mkdir(exist_ok=True)
code_executor = CodeExecutorAgent(
    name="code_executor",
    description="Run it automatically.",
    code_executor=LocalCommandLineCodeExecutor(work_dir=work_dir),
)



import asyncio

from autogen_agentchat.conditions import TextMentionTermination
from autogen_agentchat.ui import Console
from autogen_agentchat.teams import RoundRobinGroupChat, SelectorGroupChat


from autowork.autowork_ModelClient import model_client
from autowork.autowork_AssistantAgent import assistant
from autowork.autowork_UserProxyAgent import user_proxy
from autowork.autowork_CodeExecutorAgent import code_executor
from autowork.autowork_Termination import termination


def main():

    # team = RoundRobinGroupChat([assistant, user_proxy, code_executor], termination_condition=termination)

    team = SelectorGroupChat([assistant, user_proxy, code_executor], 
                             termination_condition=termination,
                             model_client=model_client,
                             allow_repeated_speaker=False,
                             selector_prompt="""You are in a role play game. The following roles are available:
    {roles}.
    Read the following conversation. Then select the next role from {participants} to play. Only return the role.

    {history}

    Read the above conversation. Then select the next role from {participants} to play. Only return the role. 
    At the beginning of the task, always starts from assistant and not from code_executor or user_proxy.
    If the code is run successfully, return the user_proxy.
    """)



    stream = team.run_stream(task=input("Hello! I am your Assistant. How could I help you?\n"))


    # Use asyncio.run(...) when running in a script.
    # await Console(stream)

    asyncio.run(Console(stream, output_stats=True))

if __name__ == "__main__":
    main()
